@@ -27,6 +27,7 @@ this object structure automatically from a .proto souce file.
 """
 
 import struct
+from itertools import izip
 
 from extprot.errors import *
 from extprot.stream import Stream, StringStream
@@ -48,7 +49,7 @@ class Type(object):
     interesting class-level methods:
 
         convert:      convert a python value to standard type representation
-        default:      retreive default value for type, if any
+        default:      get the default value for type, if any
         from_stream:  parse value from an extprot bytestream
         to_stream:    write value to an extprot bytestream
 
@@ -298,7 +299,7 @@ class Tuple(Type):
 
     @classmethod
     def to_stream(cls,value,stream):
-        values = ((t.to_stream,v) for (t,v) in zip(cls._types,value))
+        values = ((t.to_stream,v) for (t,v) in izip(cls._types,value))
         stream.write_Tuple(0,values)
 
 
@@ -311,6 +312,7 @@ class _TypedList(list):
 
     def __init__(self,type,items=()):
         self._type = type
+        items = (self._type.convert(i) for i in items)
         super(_TypedList,self).__init__(items)
 
     def __setitem__(self,key,value):
@@ -321,8 +323,16 @@ class _TypedList(list):
         super(_TypedList,self).__setitem__(key,value)
 
     def __setslice__(self,i,j,sequence):
-        items = (self._types.convert(i) for i in sequence)
-        super(_TypedList,self).__setslice__(i,j,items)
+        values = (self._types.convert(v) for v in sequence)
+        super(_TypedList,self).__setslice__(i,j,values)
+
+    def __contains__(self,value):
+        value = self._type.convert(value)
+        super(_TypedList,self).__contains__(value)
+
+    def __iter__(self):
+        for i in xrange(len(self)):
+            yield self[i]
 
     def append(self,item):
         return super(_TypedList,self).append(self._type.convert(item))
@@ -419,7 +429,7 @@ class _UnionMetaclass(type):
         #  sort them into cls._types tuple.
         #  TODO: what aboud subclasses of a Union subclass that declare
         #        additional options?  Fortunately I don't think the parser
-        #        will ever generate suc a structure.
+        #        will ever generate such a structure.
         if "_types" not in attrs:
             types = []
             is_message_union = False
@@ -456,7 +466,7 @@ class _UnionMetaclass(type):
         cls._option_from_prefix = {}
         for t in cls._types:
             # TODO: delegate this formatting to the stream somehow?
-            #       a bit tricky when we don't have one yet.
+            #       A bit tricky when we don't have one yet.
             if _issubclass(t,Option) and not t._types:
                 prefix = ((t._index << 4) | 10)
             else:
@@ -515,7 +525,6 @@ class Option(Type):
         self._values[index] = self._types[index].convert(value)
 
     def __eq__(self,other):
-        print "EQ"
         return self._values == other._values
 
     @classmethod
@@ -603,9 +612,11 @@ class Field(Type):
 
     def __get__(self,obj,type=None):
         try:
-            return obj.__dict__[self._name]
+            value = obj.__dict__[self._name]
         except KeyError:
-            return self._types[0].default()
+            value = self._types[0].default()
+            obj.__dict__[self._name] = value
+        return value
 
     def __set__(self,obj,value):
         if not self.mutable and obj._initialized:
@@ -665,7 +676,7 @@ class Message(Type):
     __metaclass__ = _MessageMetaclass
 
     #  Messages can be part of a Union, giving them a specific index.
-    #  Stand-alone messaged always have an index of zero.
+    #  Stand-alone messages always have an index of zero.
     _index = 0
 
     def __init__(self,*args,**kwds):
@@ -847,7 +858,7 @@ class Union(Type):
 class Unbound(Type):
     """Unbound type, for representing type parameters.
 
-    Attempts to use this clas in serialization raise errors.  Its intended
+    Attempts to use this class in serialization raise errors.  Its intended
     use is for individual instances to represent types that are not yet bound
     in polymorphic type declaration (i.e. to appear in cls._unbound_types).
     """
@@ -864,7 +875,7 @@ class Unbound(Type):
 class Placeholder(Type):
     """Placeholder type, for representing not-yet-defined type names.
 
-    Attempts to use this clas in serialization raise errors.  Its intended
+    Attempts to use this class in serialization raise errors.  Its intended
     use is for parsing routines to use it as a placeholder for typenames that
     haven't yet been defined.
     """
@@ -937,7 +948,7 @@ def unify_types(type1,type2):
 
     This is simplistic type unification, where instances of Unbound() in 
     one type are allowed to match concrete types in the other.  The result
-    if either None (no unification is possible) or a list of (ub,bt) pairs
+    is either None (no unification is possible) or a list of (ub,bt) pairs
     giving the necessary substitutions.
     """
     return _unify_types_rec(type1,type2,[])
@@ -1004,7 +1015,7 @@ def _unify_types_rec(type1,type2,pairs):
 def resolve_placeholders(type):
     """Iterator for resolving Placeholder() types.
 
-    This function is a generator produce (name,setter) pairs, where
+    This function is a generator producing (name,setter) pairs, where
     'name' is the name from a Placeholder instance attached to the given
     type object, and 'setter' is a function that should be called with the
     resolved value for that placeholder.  Parsing code should drive it in
